##############################################################################
#
# DavPack -- A set of patches for all things related to WebDAV
# Copyright (C) 2004 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Some portions of this module are Copyright Zope Corporatioin and
# Contributors.
# The original copyright statement is reproduced below.
#
##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: __init__.py 611 2004-11-08 17:22:10Z sidnei $
"""

import sys
import OFS
from zLOG import LOG, INFO

def log(msg):
    LOG('DavPack', INFO, msg)

from webdav.WriteLockInterface import WriteLockInterface
from webdav.common import Locked, PreconditionFailed
from zExceptions import Unauthorized, Forbidden

# This is the same value used in zhttp_collector. It should turn into
# a ZConfig setting on zope.conf when it gets merged into Zope.
LARGE_FILE = 1 << 19

def PUT(self, REQUEST, RESPONSE):
    """Create a new non-collection resource."""
    self.dav__init(REQUEST, RESPONSE)

    name = self.__name__
    parent = self.__parent__

    ifhdr = REQUEST.get_header('If', '')
    if WriteLockInterface.isImplementedBy(parent) and parent.wl_isLocked():
        if ifhdr:
            parent.dav__simpleifhandler(REQUEST, RESPONSE, col=1)
        else:
            # There was no If header at all, and our parent is locked,
            # so we fail here
            raise Locked
    elif ifhdr:
        # There was an If header, but the parent is not locked
        raise PreconditionFailed

    # SDS: Only use BODY if the file size is smaller than LARGE_FILE,
    # otherwise read LARGE_FILE bytes from the file which should be
    # enough to trigger content_type detection, and possibly enough
    # for CMF's content_type_registry too.
    #
    # Note that body here is really just used for detecting the
    # content type and figuring out the correct factory. The correct
    # file content will be uploaded on ob.PUT(REQUEST, RESPONSE) after
    # the object has been created.
    #
    # A problem I could see is content_type_registry predicates that
    # do depend on the whole file being passed here as an
    # argument. There's none by default that does this though. If they
    # really do want to look at the file, they should use
    # REQUEST['BODYFILE'] directly.

    if int(REQUEST.get('CONTENT_LENGTH') or 0) > LARGE_FILE:
        file = REQUEST['BODYFILE']
        body = file.read(LARGE_FILE)
        file.seek(0)
    else:
        body = REQUEST.get('BODY', '')

    typ = REQUEST.get_header('content-type', None)
    if typ is None:
        typ, enc = OFS.content_types.guess_content_type(name, body)

    factory = getattr(parent, 'PUT_factory', self._default_PUT_factory)
    ob = factory(name, typ, body)
    if ob is None:
        ob = self._default_PUT_factory(name, typ, body)

    # We call _verifyObjectPaste with verify_src=0, to see if the
    # user can create this type of object (and we don't need to
    # check the clipboard.
    try:
        parent._verifyObjectPaste(ob.__of__(parent), 0)
    except Unauthorized:
        raise
    except:
        raise Forbidden, sys.exc_info()[1]

    # Delegate actual PUT handling to the new object,
    # SDS: But just *after* it has been stored.
    self.__parent__._setObject(name, ob)
    ob = self.__parent__._getOb(name)
    ob.PUT(REQUEST, RESPONSE)

    RESPONSE.setStatus(201)
    RESPONSE.setBody('')
    return RESPONSE

def xml_escape(value):
    from webdav.xmltools import escape
    if not isinstance(value, basestring):
        value = unicode(value)
    if not isinstance(value, unicode):
        # XXX It really shouldn't be hardcoded to latin-1 here.
        value = value.decode('latin-1')
    value = escape(value)
    return value.encode('utf-8')

def initialize(context):
    from App.version_txt import getZopeVersion, version_txt
    zope_version = getZopeVersion()
    str_version = version_txt().split()[1].strip(',')
    if not zope_version < (2, 7, 4):
        log('Zope %s. Patches *NOT* applied.' % str_version)
    else:
        log('Zope %s. Applying patches.' % str_version)

        import webdav.xmltools
        import webdav.davcmds
        import xmltools
        webdav.xmltools.XmlParser = xmltools.XmlParser
        webdav.davcmds.XmlParser = xmltools.XmlParser

        log('Patch applied to webdav.xmltools.XmlParser.')

        from webdav.NullResource import NullResource
        NullResource.PUT = PUT

        log('Patch applied to webdav.NullResource.PUT.')

        from OFS import PropertySheets
        PropertySheets.xml_escape = xml_escape

        log('Patch applied to OFS.PropertySheets.xmlescape.')

    try:
        from Products.Archetypes import at_info
    except ImportError:
        log('Archetypes not found. *NOT* applying AT patches.')
    else:
        at_version = at_info.numversion
        str_version = at_info.version
        if not at_version < (1, 3, 1, 0):
            log('Archetypes %s. Patches *NOT* applied.' % str_version)
        else:
            log('Archetypes %s. Applying patches.' % str_version)
            from Products.DavPack import at

    try:
        from Products.CMFPlone import PloneFolder
    except ImportError:
        log('Plone not found. *NOT* applying Plone patches.')
    else:
        # XXX needs to get proper Plone version somehow.
        plone_version = (2, 0, 4, 0)
        str_version = '2.0.4'
        if not plone_version < (2, 0, 5, 0):
            log('Plone %s. Patches *NOT* applied.' % str_version)
        else:
            log('Plone %s. Applying patches.' % str_version)
            from Products.DavPack import plone
