# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: aliquot.py 758 2005-06-28 23:02:39Z sidnei $
"""

__metaclass__ = type

from datetime import date
from sqlobject import *
from sqlos import SQLOS
from sip.interfaces.aliquot import IAliquot
from sip.browser import api
from sip.descriptors import FakePropertyDescriptor

from zope.interface import implements

class MissingUnitError(ValueError):
    """A unit must be selected before setting the value"""

class Aliquot(SQLOS):

    implements(IAliquot)

    _idName='aliquot_id'
    _table='aliquot'

    _columns=[
        ForeignKey('Project', name='project_id', cascade=False),
        ForeignKey('Sample', name='sample_id', cascade=False),
        ForeignKey('VolumeUnit', name='volume_id',
                   notNull=0, cascade=False),
        ForeignKey('WeightUnit', name='weight_id',
                   notNull=0, cascade=False),
        ForeignKey('ConcentrationUnit', name='concentration_id',
                   notNull=0, cascade=False),
        ForeignKey('Container', name='container_id',
                   notNull=0, cascade=False),
        DecimalCol('volume', size=24, precision=18, notNull=0),
        DecimalCol('weight', size=24, precision=18, notNull=0),
        DecimalCol('concentration', size=24, precision=18, notNull=0),
        BoolCol('depleted', notNull=1, default=False),
        DateCol('date', notNull=1, default=date.today),
        StringCol('commit_message', notNull=1),
        BoolCol('is_low', notNull=1, default=False),
        IntCol('aliquot_count', notNull=0, default=None),
        StringCol('comment', notNull=0, default=None),
        ]

    name = FakePropertyDescriptor('name', 'id')
    label = FakePropertyDescriptor('label', 'id')

    def _get_concentration(self):
        if not self.concentration_id:
            return None
        cu = self.concentration_
        mult = cu.exponent
        value = self._SO_get_concentration()
        if value:
            value = value / mult
        return value

    def _set_concentration(self, value):
        if value is not None:
            if not self.concentration_id:
                raise MissingUnitError, 'A concentration unit must be selected'
            cu = self.concentration_
            mult = cu.exponent
            value = value * mult
        self._SO_set_concentration(value)

    def getConcentrationUnit(self):
        if not self.concentration_id:
            return None
        return self.concentration_

    def getConcentrationString(self):
        cunit = self.getConcentrationUnit()
        concentration = self.concentration
        if cunit is None or not concentration:
            return '&nbsp;'
        return '%1.2f %s' % (concentration, cunit.label)

    def _get_weight(self):
        if not self.weight_id:
            return None
        cu = self.weight_
        mult = cu.exponent
        value = self._SO_get_weight()
        if value:
            value = value / mult
        return value

    def _set_weight(self, value):
        if value is not None:
            if not self.weight_id:
                raise MissingUnitError, 'A weight unit must be selected'
            cu = self.weight_
            mult = cu.exponent
            value = value * mult
        self._SO_set_weight(value)

    def getWeightUnit(self):
        if not self.weight_id:
            return None
        return self.weight_

    def getWeightString(self):
        wunit = self.getWeightUnit()
        weight = self.weight
        if wunit is None or not weight:
            return '&nbsp;'
        return '%1.2f %s' % (weight, wunit.label)

    def _get_volume(self):
        if not self.volume_id:
            return None
        cu = self.volume_
        mult = cu.exponent
        value = self._SO_get_volume()
        if value:
            value = value / mult
        return value

    def _set_volume(self, value):
        if value is not None:
            if not self.volume_id:
                raise MissingUnitError, 'A volume unit must be selected'
            cu = self.volume_
            mult = cu.exponent
            value = value * mult
        self._SO_set_volume(value)

    def getVolumeUnit(self):
        if not self.volume_id:
            return None
        return self.volume_

    def getVolumeString(self):
        vunit = self.getVolumeUnit()
        volume = self.volume
        if vunit is None or not volume:
            return '&nbsp;'
        return '%1.2f %s' % (volume, vunit.label)

    def getLocation(self):
        if not self.container_id:
            return None
        factory = api.factory('sip.Container', context=self)
        container = factory.get(self.container_id)

        entries = []

        # The root container of a location has parent_id == 1, which
        # is the root container, used for referential integrity
        obj = container
        while obj.parent_id and obj.parent_id != 1:
            parentid = obj.parent_id
            entry = {}
            entry['title'] = obj.name
            entry['id'] = 'sip.Container.%d' % obj.id
            entries.append(entry)
            obj = factory.get(parentid)
        if not obj.parent_id:
            raise ValueError, '%s does not have a parent container' % obj.name
        factory = api.factory('sip.ContainerLocation', context=self)
        results = factory.selectBy(container_id=obj.id)
        if results.count():
            location = results[0].location_
            entry = {}
            entry['title'] = location.name
            entry['id'] = 'sip.Location.%d' % location.id
            entries.append(entry)
        factory = api.factory('sip.ContainerShipment', context=self)
        results = factory.selectBy(container_id=obj.id)
        if results.count():
            shipment = results[0].shipment_
            entry = {}
            entry['title'] = shipment.name
            entry['id'] = 'sip.Shipment.%d' % shipment.id
            entries.append(entry)
            entries.append({'title':'shipment','id':'shipment'})
        entries.reverse()
        return entries

    def isEmpty(self):
        return self.depleted

    def nextID(klass, context):
        id = api.allocateAliquotIds(context)[0]
        return id

    def getContainerType(self):
        if not self.container_: return None
        ct = self.container_.type_
        if not ct: return None
        return ct.name

    nextID = classmethod(nextID)
