# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: aliquot.py 758 2005-06-28 23:02:39Z sidnei $
"""

__metaclass__ = type

import traceback
import mx.DateTime
from datetime import date

from zope.app import zapi
from zope.event import notify
from zope.app.event.objectevent import ObjectModifiedEvent
from zope.exceptions import NotFoundError
from zope.security.proxy import ProxyFactory

from sip.csv.processor.common import Processor
from sip.browser import api
from sip.interfaces.adapter import IAugmentedSample
from sip.csv.tb import ProcessorTraceback
from zope.app.rdb.interfaces import DatabaseException

UNIT_IDS = ('weight_id', 'volume_id', 'concentration_id')

def setAliquotValues(aliquot, values):
    if not values:
        return
    for k, v in values.items():
        setattr(aliquot, k, v)
    notify(ObjectModifiedEvent(aliquot))

class BaseAliquotProcessor(Processor):
    """ Base Process that will setup the processor context """

    def process(self, context, data, request):
        """ dispatch to the subclasse's _process """
        self.context = api.getRoot(context)
        self.sample_factory = api.factory('sip.Sample', context=context)
        self.aliquot_factory = api.factory('sip.Aliquot', context=context)
        message = request.form.get('commit_message')
        if not message:
            if request.principal:
                self.message = "Edited via csv by '%s'" \
                        % request.principal.id
            else:
                self.message = "(Edited via csv - no commit message)"

        assert hasattr(self, '_process')
        return self._process(context, data, request)


class SplitAliquot(BaseAliquotProcessor):
    """Process ``split_aliquot.csv``, creating new
    aliquots"""

    def _process(self, context, data, request):
        from sip.interfaces.utilities import ISplitDerive
        factory = api.factory('sip.Aliquot', context=self.context)
        stats = {'errors':0,
                 'source_aliquots':0,
                 'destination_aliquots':0}
        src_ids = []
        src_aliquots = []
        results = {}
        src_data = data.get('source_aliquots', ())
        for src_aliquot in src_data:
            # source_aliquots must exist in the database.
            # so they are guaranteed to have a aliquot_id
            for k in ('sample_name', 'sample_id'):
                if k in src_aliquot:
                    del src_aliquot[k]
            try:
                aliquot_id = src_aliquot['aliquot_id']
                del src_aliquot['aliquot_id']
                src_aliquot['id'] = aliquot_id
                aliquot = factory.get(int(aliquot_id))
                if not aliquot:
                    raise ValueError, 'Source Aliquot %d not found' % aliquot_id
                src_ids.append(aliquot_id)
                src_aliquots.append(src_aliquot)
                if not src_aliquot.get('commit_message'):
                    src_aliquot['commit_message'] = self.message
                for c in factory._columns:
                    k = c.kw.get('name')
                    if k is None:
                        # I think this changed somewhere on the 0.6 line.
                        k = getattr(c, '_name')
                    value = src_aliquot.get(k, None)
                    if value is None:
                        src_aliquot[k] = getattr(aliquot, k)
            except Exception, exc:
                stats['errors'] += 1
                print >> self.log, 'No such aliquot %r' % aliquot_id
                traceback.print_exc(file=self.log)
                if isinstance(exc, DatabaseException):
                    raise
                continue

        dest_aliquots = []
        dest_data = data.get('destination_aliquots', ())
        for dest_aliquot in dest_data:
            # the aliquot_id in these rows must reference a
            # row in source_aliquots
            for k in ('sample_id', 'sample_name'):
                if k in dest_aliquot:
                    del dest_aliquot[k]
            try:
                aliquot_id=dest_aliquot['source_aliquot_id']
                del dest_aliquot['source_aliquot_id']
                if not dest_aliquot.get('commit_message'):
                    dest_aliquot['commit_message'] = self.message
                assert aliquot_id in src_ids
                dest_id = dest_aliquot.get('aliquot_id', None)
                if dest_id is not None:
                    del dest_aliquot['aliquot_id']
                if not dest_id:
                    dest_id = api.allocateAliquotIds(self.context)[0]
                if aliquot_id in results:
                    results[aliquot_id].append(dest_id)
                else:
                    results[aliquot_id] = [dest_id]
                dest_aliquot['id']=dest_id
                dest_aliquot['base_id']=aliquot_id
                dest_aliquots.append(dest_aliquot)
            except Exception, exc:
                stats['errors'] += 1
                print >> self.log, ('Failure in creating aliquot '
                                    'from source id %r' % aliquot_id)
                traceback.print_exc(file=self.log)
                if isinstance(exc, DatabaseException):
                    raise
                continue
        split_service = zapi.getUtility(ISplitDerive, context=self.context)
        try:
            aliquots=split_service.split(src_aliquots, dest_aliquots)
            aids = [a.id for a in aliquots]
            for source, dest in results.items():
                print >> self.log, 'Edited aliquot %s' % source
                for d in dest:
                    if int(d) in aids:
                        print >> self.log, \
                           '  Created aliquot %s from aliquot %s' % (d, source)
        except Exception, exc:
            stats['errors'] += 1
            print self.log, 'Failure during split'
            traceback.print_exc(file=self.log)
            if isinstance(exc, DatabaseException):
                raise
        else:
            stats['source_aliquots'] = len(src_aliquots)
            stats['destination_aliquots'] = len(aliquots) - len(src_aliquots)
        return stats

class ModifyAliquot(BaseAliquotProcessor):
    """Process ``modify_aliquot.csv`` making the required
    modifications to existing aliquots"""

    def _process(self, context, data, request):
        "do actual work."
        stats = {'sample':0,
                 'additional_data':0,
                 'aliquots':0,
                 'errors':0}
        context = self.context
        results = {}
        for index, sample_d in enumerate(data.get('sample', ())):
            try:
                __traceback_supplement__ = (ProcessorTraceback,
                                            sample_d, 'sample', index)
                sample_id = int(sample_d['sample_id'])
                results[sample_id] = {'additional_data':[], 'aliquots':[]}
                del sample_d['sample_id']
                sample_d['name'] = sample_d['sample_name']
                del sample_d['sample_name']
                if not sample_d.get('commit_message'):
                    sample_d['commit_message'] = self.message
                sample = self.sample_factory.get(int(sample_id))
                unchanged = True
                for k in sample_d.keys():
                    v = getattr(sample,k)
                    if type(v) is mx.DateTime.DateTimeType:
                        v = date(v.year, v.month, v.day)
                    if (v != sample_d.get(k) and k != 'commit_message'):
                        unchanged = False
                        break
                if unchanged:
                    results[sample_id]['sample'] = (
                        'Sample %s had no changes.' % sample_id)
                    continue
                __traceback_supplement__ = (ProcessorTraceback,
                                            sample_d, 'sample', index)
                sample.set(**sample_d)
                sample = api.uncontained(sample)
                notify(ObjectModifiedEvent(sample))
            except Exception, exc:
                stats['errors'] += 1
                results[sample_id]['sample'] = (
                                    'An error occurred while trying '
                                    'to edit sample %s.'
                                    % sample_id)
                traceback.print_exc(file=self.log)
                if isinstance(exc, DatabaseException):
                    raise
            else:
                stats['sample'] += 1
                results[sample_id]['sample'] = (
                                    'Successfully edited sample %s.'
                                    % sample.id)
        ad = data.get('additional_data', ())
        sample_ad = {}
        d = {}
        for sample_data in ad:
            d = sample_ad.setdefault(sample_data['sample_id'], {})
            d[sample_data['field_title_or_id']] = sample_data['field_value']
        del d, ad
        for sample_id, ad in sample_ad.items():
            if sample_id not in results:
                results[sample_id] = {}
                results[sample_id]['sample'] = ''
                results[sample_id]['aliquots'] = []
                results[sample_id]['additional_data'] = []

            try:
                sample = self._findSample(sample_id, context)
            except Exception, exc:
                stats['errors'] += 1
                print >> self.log, 'No such sample %r' % sample_id
                traceback.print_exc(file=self.log)
                if isinstance(exc, DatabaseException):
                    raise
                continue
            aug = IAugmentedSample(sample)
            schema = api.getSchemaFor(sample.type_id, api.getProject(sample))
            fields = {}
            # Additional Data uses field_title_or_id, so we
            # create a mapping that holds title -> name
            # mappings and name -> name mappings,  and then
            # we use fields[name] to get the field name,
            # be ``name`` a title or a field name.
            for name in schema.names():
                field = schema[name]
                if field.title:
                    fields[field.title] = field.__name__
                fields[field.__name__] = field.__name__
            for name, value in ad.items():
                try:
                    if getattr(aug, fields[name], None) == value:
                        results[sample_id]['additional_data'].append(
                                            '  Did not set '
                                            'field %s of sample '
                                            '%s to %s - value was '
                                            'already set.' %
                                            (name, sample_id, value))
                        continue
                    setattr(aug, fields[name], value)
                except Exception, exc:
                    stats['errors'] += 1
                    results[sample_id]['additional_data'].append(
                                        '  An error occurred'
                                        ' while trying to set field %s '
                                        'of sample %s to %r.' %
                                        (name, sample_id, value))
                    traceback.print_exc(file=self.log)
                    if isinstance(exc, DatabaseException):
                        raise
                    continue
                else:
                    stats['additional_data'] += 1
                    results[sample_id]['additional_data'].append(
                                        '  Sucessfully set '
                                        'field %s of sample '
                                        '%s to %s.' %
                                        (name, sample_id, value))
        d_aliquots = data.get('aliquots', None)
        if d_aliquots:
            for index, aliquot_d in enumerate(d_aliquots):
                __traceback_supplement__ = (ProcessorTraceback,
                                            aliquot_d, 'aliquots', index)
                sample_id = aliquot_d['sample_id']
                if sample_id not in results:
                    results[sample_id] = {}
                    results[sample_id]['sample'] = ''
                    results[sample_id]['aliquots'] = []
                    results[sample_id]['additional_data'] = []
                try:
                    aliquot_id = aliquot_d['aliquot_id']
                    del aliquot_d['aliquot_id']
                    if not aliquot_d.get('comment'):
                        aliquot_d['comment'] = None
                    if not aliquot_d.get('commit_message'):
                        aliquot_d['commit_message'] = self.message
                    # we want to remove the values so we can be sure to set
                    # them after setting the units
                    values = {}
                    for k in ('concentration', 'weight', 'volume'):
                        value = aliquot_d[k]
                        if value:
                            values[k] = value
                            aliquot_d[k] = None
                    if aliquot_id:
                        aliquot = self.aliquot_factory.get(int(aliquot_id))
                        unchanged = True
                        for k in aliquot_d.keys():
                            if (getattr(aliquot, k) != aliquot_d.get(k) and
                                k != 'commit_message' or (k in values and
                                getattr(aliquot, k) != values[k])):
                                unchanged = False
                                break
                        if unchanged:
                            results[sample_id]['aliquots'].append(
                                'Aliquot %s had no changes.' % aliquot_id)
                            continue
                        __traceback_supplement__ = (ProcessorTraceback,
                                                    aliquot_d, 'aliquots',
                                                    index)
                        aliquot.set(**aliquot_d)
                        # We are doing this here so that __parent__
                        # and __name__ get cleared, otherwise their
                        # state is unkown and worse, they are still
                        # referenced from the cache, which yelds to a
                        # random error on publishing the event
                        aliquot = api.uncontained(aliquot)
                        notify(ObjectModifiedEvent(aliquot))
                        setAliquotValues(aliquot, values)
                        log_msg = '  Successfully edited aliquot %s.'
                    else:
                        # no id - create it
                        aliquot_d['id'] = api.allocateAliquotIds(context)[0]
                        aliquot = self.aliquot_factory(**aliquot_d)
                        setAliquotValues(aliquot, values)
                        log_msg = '  Successfully added aliquot %s.'
                except Exception, exc:
                    stats['errors'] += 1
                    results[sample_id]['aliquots'].append(
                                        '  An error occurred while trying '
                                        'to edit aliquot %s.'
                                        % aliquot_id)
                    traceback.print_exc(file=self.log)
                    if isinstance(exc, DatabaseException):
                        raise
                else:
                    stats['aliquots'] += 1
                    results[sample_id]['aliquots'].append(log_msg % aliquot.id)
        for v in results.values():
            print >> self.log, v['sample']
            for ad in v['additional_data']:
                print >> self.log, ad
            for al in v['aliquots']:
                print >> self.log, al
        return stats

    def _findSample(self, sample_id, context):
        sample = self.sample_factory.get(int(sample_id))
        return api.contained(sample, appRoot=context)
