# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: test_aliquot.py 749 2005-06-28 22:21:49Z sidnei $
"""

import unittest

from zope.app import zapi
from zope.app.container.interfaces import IReadContainer
from zope.security.proxy import ProxyFactory
from zope.event import notify
from zope.app.event.objectevent import ObjectModifiedEvent

from sip.ftests.base import Base
from sip.interfaces.adapter import IZopeAliquot, \
     ISampleWithAliquots
from sip.interfaces.utilities import ISplitDerive

class TestAliquot(Base):
    level = 2
    use_test_user = False

    def testAddAliquot(self):
        project = self._addProject()
        aliquots = zapi.traverse(project,'aliquots')
        tech = self._addTechnician()
        stype = self._addSampleType(project)
        sample = self._addSample(project, stype.id, tech.id)
        self.assertEquals(len(aliquots),0)
        aliquot = self._addAliquot(project, sample.id)
        self.assertEquals(len(aliquots), 1)

    def testMoveAliquot(self):
        project = self._addProject()
        aliquots = zapi.traverse(project,'aliquots')
        tech = self._addTechnician()
        stype = self._addSampleType(project)
        sample = self._addSample(project, stype.id, tech.id)
        ct = self._addContainerType()
        location = self._addLocation(type_id=ct.id)
        container1 = self._addContainer(location, ct.id)
        container2 = self._addContainer(location, ct.id, name='Container 2')
        ad1 = IReadContainer(container1)
        ad2 = IReadContainer(container2)
        self.assertEquals(len(ad1), 0)
        self.assertEquals(len(ad2), 0)
        self.assertEquals(len(aliquots), 0)
        aliquot = self._addAliquot(project, sample.id, container_id=container1.id)
        self.assertEquals(len(ad1), 1)
        self.assertEquals(len(ad2), 0)
        self.assertEquals(len(aliquots), 1)
        aliquot.container_id = container2.id
        notify(ObjectModifiedEvent(aliquot))
        self.assertEquals(len(ad1), 0)
        self.assertEquals(len(ad2), 1)
        aliquot = self._addAliquot(project, sample.id, container_id=container2.id)
        self.assertEquals(len(aliquots), 2)
        self.assertEquals(len(ad1), 0)
        self.assertEquals(len(ad2), 2)

    def testAliquotSplit(self):
        from sip.aliquot import Aliquot
        vunit = self._addVolumeUnit()
        wunit = self._addWeightUnit()
        project = self._addProject()
        tech = self._addTechnician()
        stype = self._addSampleType(project)
        sample = self._addSample(project, stype.id, tech.id)
        aliquot = self._addAliquot(project, sample.id)
        ss = zapi.getUtility(ISplitDerive, context=project)
        base_aliquots = []
        new_aliquots = []
        aliquot_rec = {}
        aliquot_rec['id'] = aliquot.id
        aliquot_rec['volume'] = 5.0
        aliquot_rec['volume_unit'] = vunit.id
        aliquot_rec['weight'] = 6.0
        aliquot_rec['weight_unit'] = wunit.id
        aliquot_rec['concentration'] = None
        aliquot_rec['concentration_unit'] = None
        aliquot_rec['container_id'] = None
        aliquot_rec['aliquot_count'] = None
        aliquot_rec['comment'] = None
        base_aliquots.append(aliquot_rec)
        for i in range(2):
            rec = {}
            rec['base_id'] = aliquot.id
            rec['id'] = Aliquot.nextID(sample)
            rec['volume'] = 2.0
            rec['volume_unit'] = vunit.id
            rec['weight'] = 2.5
            rec['weight_unit'] = wunit.id
            rec['concentration'] = None
            rec['concentration_unit'] = None
            rec['container_id'] = None
            rec['aliquot_count'] = None
            rec['comment'] = None
            new_aliquots.append(rec)
        aliquots = ss.split(base_aliquots, new_aliquots)
        self.failUnless(len(aliquots) == 3)
        sid = aliquot.sample_id
        self.failUnless(aliquots[0].sample_id == sid
                        and aliquots[1].sample_id == sid
                        and aliquots[2].sample_id == sid)

    def testAliquotUnits(self):
        project = self._addProject()
        cunit = self._addConcentrationUnit()
        vunit = self._addVolumeUnit()
        wunit = self._addWeightUnit()
        tech = self._addTechnician()
        stype = self._addSampleType(project)
        sample = self._addSample(project, stype.id, tech.id)
        aliquot = self._addAliquot(project, sample.id, volume_id = vunit.id,
                                   concentration_id = cunit.id,
                                   weight_id = wunit.id)
        ncunit = aliquot.getConcentrationUnit()
        nvunit = aliquot.getVolumeUnit()
        nwunit = aliquot.getWeightUnit()

        self.failUnless(cunit.id == ncunit.id)
        self.failUnless(vunit.id == nvunit.id)
        self.failUnless(wunit.id == nwunit.id)

    def testGetLocation(self):
        project = self._addProject()
        tech = self._addTechnician()
        stype = self._addSampleType(project)
        sample = self._addSample(project, stype.id, tech.id)
        ct = self._addContainerType()
        box = self._addContainerType(name=u'box',root=False)
        loc = self._addLocation(ct.id)
        con = self._addContainer(loc, box.id)
        aliquot = self._addAliquot(project, sample.id, container_id = con.id)
        loc_info = aliquot.getLocation()
        self.failUnless(len(loc_info) == 2)

    def testAdaptAliquotToSampleWithAliquots(self):
        self.project = project = self._addProject()
        tech = self._addTechnician()
        stype = self._addSampleType(project)
        sample = self._addSample(project, stype.id, tech.id)
        aliquot = self._addAliquot(project, sample.id)
        adapted = ISampleWithAliquots(aliquot)
        self.failUnless(adapted.aliquots)
        self.failUnless(len(adapted.aliquots) == 1)
        self.failUnless(adapted.aliquots.values()[0].id == aliquot.id)
        self.failUnless(adapted.id == sample.id)

    def testAdaptAliquotToIZopeAliquot(self):
        project = self._addProject()
        cunit = self._addConcentrationUnit()
        vunit = self._addVolumeUnit()
        wunit = self._addWeightUnit()
        tech = self._addTechnician()
        stype = self._addSampleType(project)
        sample = self._addSample(project, stype.id, tech.id)
        ct = self._addContainerType()
        box = self._addContainerType(name=u'box',root=False)
        loc = self._addLocation(ct.id)
        con = self._addContainer(loc, box.id)
        aliquot = self._addAliquot(project, sample.id, volume_id = vunit.id,
                                   concentration_id = cunit.id,
                                   weight_id = wunit.id,
                                   container_id = con.id)
        ncunit = aliquot.getConcentrationUnit()
        nvunit = aliquot.getVolumeUnit()
        nwunit = aliquot.getWeightUnit()
        adapted = IZopeAliquot(aliquot)

        self.failIf(adapted.volume_unit is None)
        vpath = zapi.getPath(adapted.volume_unit)
        expected = '/sip/volumeunits/sip.VolumeUnit.%s' % nvunit.id
        self.failUnless(vpath == expected)

        self.failIf(adapted.concentration_unit is None)
        cpath = zapi.getPath(adapted.concentration_unit)
        expected = '/sip/concentrationunits/sip.ConcentrationUnit.%s' % ncunit.id
        self.failUnless(cpath == expected)

        self.failIf(adapted.weight_unit is None)
        wpath = zapi.getPath(adapted.weight_unit)
        expected = '/sip/weightunits/sip.WeightUnit.%s' % nwunit.id
        self.failUnless(wpath == expected)

        self.failIf(adapted.container is None)
        conpath = zapi.getPath(adapted.container)
        expected = '/sip/locations/sip.Location.%s/./sip.Container.%s' % (loc.id,
                                                                   con.id)
        self.failUnless(conpath == expected, "%s != %s" % (conpath, expected))

        self.failIf(adapted.sample is None)
        spath = zapi.getPath(adapted.sample)
        expected = '/sip/projects/sip.Project.%s/samples/sip.Sample.%s' % (
            project.id, sample.id)
        self.failUnless(spath == expected)

        self.failIf(adapted.project is None)
        ppath = zapi.getPath(adapted.project)
        expected = '/sip/projects/sip.Project.%s' % project.id
        self.failUnless(ppath == expected)

    def testEditAliquotView(self):
        project = self._addProject()
        aliquots = zapi.traverse(project,'aliquots')
        tech = self._addTechnician()
        stype = self._addSampleType(project)
        sample = self._addSample(project, stype.id, tech.id)
        self.assertEquals(len(aliquots),0)
        aliquot = self._addAliquot(project, sample.id)
        self.assertEquals(len(aliquots), 1)
        edit = zapi.getView(aliquot, 'edit.html', self.request)
        self.failUnless(edit())

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestAliquot))
    return suite

if __name__ == '__main__':
    unittest.main()
