# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: test_query.py 749 2005-06-28 22:21:49Z sidnei $
"""

import unittest

from datetime import date
from zope.app import zapi
from zope.schema import Text, Int, Float, Bool
from zope.security.proxy import ProxyFactory
from sip.schema.interfaces import IMutableSchema

from sip.browser import api
from sip.ftests.base import Base
from sip.query import QueryCommand, SearchType, Query, SortOn
from sip.interfaces.utilities import IQuery

class TestQuery(Base):
    level = 3
    use_test_user = False

    def testAliquotQueryCommand(self):
        sip = zapi.traverse(self.rootFolder, 'sip')
        project = self._addProject()
        tech = self._addTechnician()
        stype = self._addSampleType(project)
        sample = self._addSample(project, stype.id, tech.id)
        aliquot = self._addAliquot(project, sample.id )
        cmd = QueryCommand(SearchType.aliquot, labels=[aliquot.id],
                           project_ids = [project.id])
        results = cmd.execute(sip)
        self.failUnless(len(results)==1)
        res_sample = results[0]
        self.failUnless(res_sample.id == sample.id)
        aliquots = results[0].aliquots.values()
        self.failUnless(len(aliquots)==1)
        res_aliquot = aliquots[0]
        self.failUnless(res_aliquot.id == aliquot.id)

    def testSubjectIDQueryCommand(self):
        project = self._addProject()
        sip = zapi.traverse(self.rootFolder, 'sip')
        tech = self._addTechnician()
        stype = self._addSampleType(project)
        sample = self._addSample(project, stype.id, tech.id,
                                 subject_id='a123')
        aliquots = []
        aliquots.append(self._addAliquot(project, sample.id))
        aliquots.append(self._addAliquot(project, sample.id))
        alids = [a.id for a in aliquots]
        cmd = QueryCommand(SearchType.project, subject_ids=['a123'],
                           project_ids = [project.id], id_type='subject_id')
        results = cmd.execute(sip)
        self.failUnless(len(results)==1)
        res_sample = results[0]
        self.failUnless(res_sample.id == sample.id)
        res_aliquots = results[0].aliquots.values()
        self.failUnless(len(res_aliquots)==2)
        self.failUnless(res_aliquots[0].id != res_aliquots[1].id)
        self.failUnless(res_aliquots[0].id in alids
                        and res_aliquots[1].id in alids)

    def testOtherIDQueryCommand(self):
        project = self._addProject()
        sip = zapi.traverse(self.rootFolder, 'sip')
        tech = self._addTechnician()
        stype = self._addSampleType(project)
        sample = self._addSample(project, stype.id, tech.id,
                                 other_id='a123')
        aliquots = []
        aliquots.append(self._addAliquot(project, sample.id))
        aliquots.append(self._addAliquot(project, sample.id))
        alids = [a.id for a in aliquots]
        cmd = QueryCommand(SearchType.otherid, ids=['a123'],
                           project_ids = [project.id])
        results = cmd.execute(sip)
        self.failUnless(len(results)==1)
        res_sample = results[0]
        self.failUnless(res_sample.id == sample.id)
        res_aliquots = results[0].aliquots.values()
        self.failUnless(len(res_aliquots)==2)
        self.failUnless(res_aliquots[0].id != res_aliquots[1].id)
        self.failUnless(res_aliquots[0].id in alids
                        and res_aliquots[1].id in alids)

    def testSampleNameQueryCommand(self):
        project = self._addProject()
        sip = zapi.traverse(self.rootFolder, 'sip')
        tech = self._addTechnician()
        stype = self._addSampleType(project)
        sample = self._addSample(project, stype.id, tech.id,
                                 name='a123')
        aliquots = []
        aliquots.append(self._addAliquot(project, sample.id))
        aliquots.append(self._addAliquot(project, sample.id))
        alids = [a.id for a in aliquots]
        cmd = QueryCommand(SearchType.samplename, names=['a123'],
                           project_ids = [project.id])
        results = cmd.execute(sip)
        self.failUnless(len(results)==1)
        res_sample = results[0]
        self.failUnless(res_sample.id == sample.id)
        res_aliquots = results[0].aliquots.values()
        self.failUnless(len(res_aliquots)==2)
        self.failUnless(res_aliquots[0].id != res_aliquots[1].id)
        self.failUnless(res_aliquots[0].id in alids
                        and res_aliquots[1].id in alids)

    def testSampleTypeQueryCommand(self):
        project = self._addProject()
        sip = zapi.traverse(self.rootFolder, 'sip')
        tech = self._addTechnician()
        stype = self._addSampleType(project)
        sample = self._addSample(project, stype.id, tech.id,
                                 name='a123')
        aliquots = []
        aliquots.append(self._addAliquot(project, sample.id))
        aliquots.append(self._addAliquot(project, sample.id))
        alids = [a.id for a in aliquots]
        cmd = QueryCommand(SearchType.project, type_ids=[stype.id],
                           project_ids = [project.id])
        results = cmd.execute(sip)
        self.failUnless(len(results)==1)
        res_sample = results[0]
        self.failUnless(res_sample.id == sample.id)
        res_aliquots = results[0].aliquots.values()
        self.failUnless(len(res_aliquots)==2)
        self.failUnless(res_aliquots[0].id != res_aliquots[1].id)
        self.failUnless(res_aliquots[0].id in alids
                        and res_aliquots[1].id in alids)

    def testProjectQueryCommand(self):
        project = self._addProject()
        sip = zapi.traverse(self.rootFolder, 'sip')
        tech = self._addTechnician()
        stype = self._addSampleType(project)
        sample = self._addSample(project, stype.id, tech.id,
                                 name='a123')
        aliquots = []
        aliquots.append(self._addAliquot(project, sample.id))
        aliquots.append(self._addAliquot(project, sample.id))
        alids = [a.id for a in aliquots]
        cmd = QueryCommand(SearchType.project, project_ids=[project.id])
        results = cmd.execute(sip)
        self.failUnless(len(results)==1)
        res_sample = results[0]
        self.failUnless(res_sample.id == sample.id)
        res_aliquots = results[0].aliquots.values()
        self.failUnless(len(res_aliquots)==2)
        self.failUnless(res_aliquots[0].id != res_aliquots[1].id)
        self.failUnless(res_aliquots[0].id in alids
                        and res_aliquots[1].id in alids)

    def testCollectionDateQuery(self):
        project = self._addProject()
        sip = zapi.traverse(self.rootFolder, 'sip')
        tech = self._addTechnician()
        stype = self._addSampleType(project)
        aliquots = []
        sample = self._addSample(project, stype.id, tech.id,
                                 collection_date = date(2003, 10, 25),
                                 name='a123')
        sample2 = self._addSample(project, stype.id, tech.id,
                                 collection_date = date(2003, 10, 24),
                                 name='a1234')
        aliquots.append(self._addAliquot(project, sample.id))
        aliquots.append(self._addAliquot(project, sample2.id))
        cmd = QueryCommand(SearchType.project, project_ids=[project.id],
                           collect_dates={'start_date':'10/25/2003',
                                          'end_date':'10/25/2003'})
        results = cmd.execute(sip)
        self.failUnless(len(results)==1)

    def testDateEnteredQuery(self):
        project = self._addProject()
        sip = zapi.traverse(self.rootFolder, 'sip')
        tech = self._addTechnician()
        stype = self._addSampleType(project)
        aliquots = []
        sample = self._addSample(project, stype.id, tech.id,
                                 date_generated = date(2003, 10, 25),
                                 name='a123')
        sample2 = self._addSample(project, stype.id, tech.id,
                                 date_generated = date(2003, 10, 24),
                                 name='a1234')
        aliquots.append(self._addAliquot(project, sample.id))
        aliquots.append(self._addAliquot(project, sample2.id))
        cmd = QueryCommand(SearchType.project, project_ids=[project.id],
                           entered_dates={'start_date':'10/25/2003',
                                          'end_date':'10/25/2003'})
        results = cmd.execute(sip)
        self.failUnless(len(results)==1)

    def testAddDataTextQuery(self):
        project = self._addProject()
        stype = self._addSampleType(project)
        schema = self._addSchema(project, stype)
        schema = IMutableSchema(schema)
        tech = self._addTechnician()
        sample = self._addSample(project, stype.id, tech.id,
                                 name='sample1')
        sample2 = self._addSample(project, stype.id, tech.id,
                                  name='sample2')
        field = Text(title=u'Test Id', required=False)
        schema.addField(u'test_id', field)
        field = schema.get('test_id')
        self._addSampleData(sample, project, 'test_id', 'abc123', stype)
        self._addSampleData(sample2, project, 'test_id', '123abc', stype)
        field_id = api.getFieldId(project, sample.id, 'test_id')
        info_querys = [{'field_id':field_id,'op':'=','value':'abc123'}]
        cmd = QueryCommand(SearchType.addinfo, info_querys = info_querys,
                           project_ids = [project.id])
        results = cmd.execute(self.sip)
        self.failUnless(len(results)==1)

    def testAddDataIntEqualsQuery(self):
        project = self._addProject()
        stype = self._addSampleType(project)
        schema = self._addSchema(project, stype)
        schema = IMutableSchema(schema)
        tech = self._addTechnician()
        sample = self._addSample(project, stype.id, tech.id,
                                 name='sample1')
        sample2 = self._addSample(project, stype.id, tech.id,
                                  name='sample2')
        sample3 = self._addSample(project, stype.id, tech.id,
                                  name='sample3')
        field = Int(title=u'HOL', required=False)
        schema.addField(u'hol', field)
        field = Text(title=u'test_id', required=False)
        schema.addField(u'test_id', field)

        self._addSampleData(sample, project, 'hol', '24', stype)
        self._addSampleData(sample, project, 'test_id', 'test123', stype)
        self._addSampleData(sample2, project, 'hol', '12', stype)
        self._addSampleData(sample3, project, 'hol', '6', stype)

        field_id = api.getFieldId(project, sample.id, 'hol')
        info_querys = [{'field_id':field_id,'op':'=','value':'24'}]
        cmd = QueryCommand(SearchType.addinfo, info_querys = info_querys,
                           project_ids = [project.id])
        results = cmd.execute(self.sip)
        self.failUnless(len(results)==1)
        self.failUnless(results[0].id == sample.id)

    def testAddDataIntGTOREqualsQuery(self):
        project = self._addProject()
        stype = self._addSampleType(project)
        schema = self._addSchema(project, stype)
        schema = IMutableSchema(schema)
        tech = self._addTechnician()
        sample = self._addSample(project, stype.id, tech.id,
                                 name='sample1')
        sample2 = self._addSample(project, stype.id, tech.id,
                                  name='sample2')
        sample3 = self._addSample(project, stype.id, tech.id,
                                  name='sample3')
        field = Int(title=u'HOL', required=False)
        schema.addField(u'hol', field)
        field = Text(title=u'test_id', required=False)
        schema.addField(u'test_id', field)

        self._addSampleData(sample, project, 'hol', '24', stype)
        self._addSampleData(sample, project, 'test_id', 'test123', stype)
        self._addSampleData(sample2, project, 'hol', '12', stype)
        self._addSampleData(sample3, project, 'hol', '6', stype)
        field_id = api.getFieldId(project, sample.id, 'hol')
        info_querys = [{'field_id':field_id,'op':'>=','value':'22'}]
        cmd = QueryCommand(SearchType.addinfo, info_querys = info_querys,
                           project_ids = [project.id])
        results = cmd.execute(self.sip)
        self.failUnless(len(results)==1)
        self.failUnless(results[0].id == sample.id)

    def testAddDataIntLTOREqualsQuery(self):
        project = self._addProject()
        stype = self._addSampleType(project)
        schema = self._addSchema(project, stype)
        schema = IMutableSchema(schema)
        tech = self._addTechnician()
        sample = self._addSample(project, stype.id, tech.id,
                                 name='sample1')
        sample2 = self._addSample(project, stype.id, tech.id,
                                  name='sample2')
        sample3 = self._addSample(project, stype.id, tech.id,
                                  name='sample3')
        field = Int(title=u'HOL', required=False)
        schema.addField(u'hol', field)
        field = Text(title=u'test_id', required=False)
        schema.addField(u'test_id', field)

        self._addSampleData(sample, project, 'hol', '24', stype)
        self._addSampleData(sample, project, 'test_id', 'test123', stype)
        self._addSampleData(sample2, project, 'hol', '12', stype)
        self._addSampleData(sample3, project, 'hol', '6', stype)
        field_id = api.getFieldId(project, sample.id, 'hol')
        info_querys = [{'field_id':field_id,'op':'<=','value':'10'}]
        cmd = QueryCommand(SearchType.addinfo, info_querys = info_querys,
                           project_ids = [project.id])
        results = cmd.execute(self.sip)
        self.failUnless(len(results)==1)
        self.failUnless(results[0].id == sample3.id)

    def testAddDataFloatEqualsQuery(self):
        project = self._addProject()
        stype = self._addSampleType(project)
        schema = self._addSchema(project, stype)
        schema = IMutableSchema(schema)
        tech = self._addTechnician()
        sample = self._addSample(project, stype.id, tech.id,
                                 name='sample1')
        sample2 = self._addSample(project, stype.id, tech.id,
                                  name='sample2')
        sample3 = self._addSample(project, stype.id, tech.id,
                                  name='sample3')
        field = Float(title=u'HOL', required=False)
        schema.addField(u'hol', field)
        field = Text(title=u'test_id', required=False)
        schema.addField(u'test_id', field)

        self._addSampleData(sample, project, 'hol', '24.5', stype)
        self._addSampleData(sample, project, 'test_id', 'test123', stype)
        self._addSampleData(sample2, project, 'hol', '24.25', stype)
        self._addSampleData(sample3, project, 'hol', '24', stype)

        field_id = api.getFieldId(project, sample.id, 'hol')
        info_querys = [{'field_id':field_id,'op':'=','value':'24.5'}]
        cmd = QueryCommand(SearchType.addinfo, info_querys = info_querys,
                           project_ids = [project.id])
        results = cmd.execute(self.sip)
        self.failUnless(len(results)==1)
        self.failUnless(results[0].id == sample.id)

    def testAddDataFloatGTOREqualsQuery(self):
        project = self._addProject()
        stype = self._addSampleType(project)
        schema = self._addSchema(project, stype)
        schema = IMutableSchema(schema)
        tech = self._addTechnician()
        sample = self._addSample(project, stype.id, tech.id,
                                 name='sample1')
        sample2 = self._addSample(project, stype.id, tech.id,
                                  name='sample2')
        sample3 = self._addSample(project, stype.id, tech.id,
                                  name='sample3')
        field = Float(title=u'HOL', required=False)
        schema.addField(u'hol', field)
        field = Text(title=u'test_id', required=False)
        schema.addField(u'test_id', field)

        self._addSampleData(sample, project, 'hol', '24.75', stype)
        self._addSampleData(sample, project, 'test_id', 'test123', stype)
        self._addSampleData(sample2, project, 'hol', '24.5', stype)
        self._addSampleData(sample3, project, 'hol', '24.25', stype)
        field_id = api.getFieldId(project, sample.id, 'hol')
        info_querys = [{'field_id':field_id,'op':'>=','value':'24.55'}]
        cmd = QueryCommand(SearchType.addinfo, info_querys = info_querys,
                           project_ids = [project.id])
        results = cmd.execute(self.sip)
        self.failUnless(len(results)==1)
        self.failUnless(results[0].id == sample.id)

    def testAddDataFloatLTOREqualsQuery(self):
        project = self._addProject()
        stype = self._addSampleType(project)
        schema = self._addSchema(project, stype)
        schema = IMutableSchema(schema)
        tech = self._addTechnician()
        sample = self._addSample(project, stype.id, tech.id,
                                 name='sample1')
        sample2 = self._addSample(project, stype.id, tech.id,
                                  name='sample2')
        sample3 = self._addSample(project, stype.id, tech.id,
                                  name='sample3')
        field = Float(title=u'HOL', required=False)
        schema.addField(u'hol', field)
        field = Text(title=u'test_id', required=False)
        schema.addField(u'test_id', field)

        self._addSampleData(sample, project, 'hol', '24.75', stype)
        self._addSampleData(sample, project, 'test_id', 'test123', stype)
        self._addSampleData(sample2, project, 'hol', '24.5', stype)
        self._addSampleData(sample3, project, 'hol', '24.25', stype)
        field_id = api.getFieldId(project, sample.id, 'hol')
        info_querys = [{'field_id':field_id,'op':'<=','value':'24.45'}]
        cmd = QueryCommand(SearchType.addinfo, info_querys = info_querys,
                           project_ids = [project.id])
        results = cmd.execute(self.sip)
        self.failUnless(len(results)==1)
        self.failUnless(results[0].id == sample3.id)

    def testAddDataBoolEqualsQuery(self):
        project = self._addProject()
        stype = self._addSampleType(project)
        schema = self._addSchema(project, stype)
        schema = IMutableSchema(schema)
        tech = self._addTechnician()
        sample = self._addSample(project, stype.id, tech.id,
                                 name='sample1')
        sample2 = self._addSample(project, stype.id, tech.id,
                                  name='sample2')
        sample3 = self._addSample(project, stype.id, tech.id,
                                  name='sample3')
        field = Bool(title=u'Is Good', required=False)
        schema.addField(u'is_good', field)
        field = Text(title=u'test_id', required=False)
        schema.addField(u'test_id', field)

        self._addSampleData(sample, project, 'is_good', '1', stype)
        self._addSampleData(sample, project, 'test_id', 'test123', stype)
        self._addSampleData(sample2, project, 'is_good', '0', stype)
        self._addSampleData(sample3, project, 'is_good', '0', stype)

        field_id = api.getFieldId(project, sample.id, 'is_good')
        info_querys = [{'field_id':field_id,'op':'=','value':'1'}]
        cmd = QueryCommand(SearchType.addinfo, info_querys = info_querys,
                           project_ids = [project.id])
        results = cmd.execute(self.sip)
        self.failUnless(len(results)==1)
        self.failUnless(results[0].id == sample.id)


    def _addSampleData(self, sample, project, fname, data, sample_type=None):
        stname = ''
        if sample_type is not None:
            stname = sample_type.name
        field_id = api.getFieldId(project, sample.id, fname, stname)
        factory = api.factory('sip.AdditionalSampleData', context=project)
        kw = {}
        kw['sample_id'] = sample.id
        kw['schema_field_id'] = field_id
        kw['value'] = data
        factory(**kw)


    def _addSchema(self, project, sample_type):
        schemas = zapi.traverse(project, 'schemas')
        adding = zapi.getView(schemas, '+', self.request)
        adding = zapi.getView(adding, 'add_sample_type_schema.html',
                              self.request)

        kw = {}
        kw['project_id'] = project.id
        kw['sample_type_id'] = sample_type.id

        schema = adding.create(**kw)
        return schemas['sip.SampleTypeSchema.%d' % schema.id]


    def _querySetup(self):
        projects = {}
        projects['project1'] = {}
        project1 = self._addProject(name='Project1',
                                    require_commit_message=True)
        projects['project1']['project'] = project1
        projects['project2'] = {}
        project2 = self._addProject(name='Project2')
        projects['project2']['project'] = project2
        self.projects = projects
        stypes1 = []
        tech = self._addTechnician()
        stypes1.append(self._addSampleType(project1))
        stypes1.append(self._addSampleType(project1, name='DNA'))
        projects['project1']['stypes'] = stypes1
        stypes2 = []
        stypes2.append(self._addSampleType(project2))
        stypes2.append(self._addSampleType(project2,name='Serum'))
        projects['project2']['stypes'] = stypes2
        samples1 = []
        samples1.append(self._addSample(project1, stypes1[0].id,
                                        tech.id, name = u'sample11',
                                        other_id = u'other8723',
                                        date_generated=date(2003, 10, 15),
                                        collection_date=date(2003, 10, 11),
                                        subject_id=u'sub982346'))
        samples1.append(self._addSample(project1, stypes1[0].id,
                                        tech.id, name=u'sample05',
                                        other_id=u'other9483',
                                        subject_type_id=self.other.id,
                                        date_generated=date(2003, 8, 24),
                                        collection_date=date(2003, 8, 5),
                                        subject_id=u'sub585946'))
        samples1.append(self._addSample(project1, stypes1[1].id,
                                        tech.id, name = u'sample09',
                                        other_id = u'other3864',
                                        date_generated=date(2002, 12, 23),
                                        collection_date=date(2003, 12, 20),
                                        subject_id=u'sub982342'))
        projects['project1']['samples'] = samples1
        samples2 = []
        samples2.append(self._addSample(project2, stypes2[0].id,
                                        tech.id, name=u'sample05',
                                        other_id=u'other9482',
                                        date_generated=date(2003, 10, 23),
                                        collection_date=date(2003, 10, 5),
                                        subject_id=u'sub64523'))
        projects['project2']['samples'] = samples2
        aliquots1 = []
        aliquots1.append(self._addAliquot(project1, samples1[0].id))
        aliquots1.append(self._addAliquot(project1, samples1[0].id))
        aliquots1.append(self._addAliquot(project1, samples1[0].id))
        aliquots1.append(self._addAliquot(project1, samples1[1].id))
        projects['project1']['aliquots'] = aliquots1
        aliquots2 = []
        aliquots2.append(self._addAliquot(project2, samples2[0].id))

    def testQueryService(self):
        project = self._addProject()
        sip = zapi.traverse(self.rootFolder, 'sip')
        tech = self._addTechnician()
        stype = self._addSampleType(project)
        sample = self._addSample(project, stype.id, tech.id,
                                 name='a123')
        aliquots = []
        aliquots.append(self._addAliquot(project, sample.id ))
        aliquots.append(self._addAliquot(project, sample.id ))
        alids = [a.id for a in aliquots]
        cmds = []
        cmds.append(QueryCommand(SearchType.project, project_ids=[project.id]))
        cmds.append(QueryCommand(SearchType.aliquot, labels=[aliquots[0].id],
                                 project_ids=[project.id]))
        query = Query(cmds)
        qs = zapi.getUtility(IQuery, context=sip)
        results = qs.search(query)
        results = results.getSamples()
        self.failUnless(len(results)==1)
        res_sample = results[0]
        self.failUnless(res_sample.id == sample.id)
        res_aliquots = results[0].aliquots.values()
        self.failUnless(len(res_aliquots)==1)
        self.failUnless(res_aliquots[0].id == aliquots[0].id)

    def testQueryCSV(self):
        query_view = self._setupQueryView()
        query_view.request.form['export_to_csv'] = True
        self.failUnless(query_view.validate() == query_view.export_csv())

    def testSampleNameSort(self):
        self._querySetup()
        sip = zapi.traverse(self.rootFolder, 'sip')
        cmds = []
        project = self.projects['project1']['project']
        cmds.append(QueryCommand(SearchType.project, project_ids=[project.id]))
        query = Query(cmds, sorton = SortOn.samplename)
        qs = zapi.getUtility(IQuery, context=sip)
        results = qs.search(query)
        results = results.getSamples()
        self.failUnless(len(results)==3)
        sample_names = [r.name for r in results]
        self.failUnless(sample_names==['sample05','sample09','sample11'],
                        sample_names)

    def testDateGeneratedSort(self):
        self._querySetup()
        sip = zapi.traverse(self.rootFolder, 'sip')
        cmds = []
        project = self.projects['project1']['project']
        cmds.append(QueryCommand(SearchType.project, project_ids=[project.id]))
        query = Query(cmds, sorton = SortOn.dategen)
        qs = zapi.getUtility(IQuery, context=sip)
        results = qs.search(query)
        results = results.getSamples()
        dates = [r.date_generated.strftime('%m/%d/%Y') for r in results]
        self.failUnless(len(results)==3)
        self.failUnless(dates==['12/23/2002','08/24/2003','10/15/2003'])

    def _setupQueryView(self):
        self._querySetup()
        request = self.request
        project1 = self.projects['project1']['project']
        project2 = self.projects['project2']['project']
        samples = self.projects['project1']['samples']
        aliquots = self.projects['project1']['aliquots']
        view = zapi.getView(project1, 'search.html', request)
        view.request.form['project_ids'] = [project1.id]
        view.request.form['sample_types'] = [samples[0].type_id]
        #view.request.form['sample_names'] = 'testSample'
        #view.request.form['subject_ids'] = u'sub585946'
        #view.request.form['other_ids'] = 'other'
        view.request.form['order_by'] = 'dategen'
        view.request.form['search'] = True
        return view

    def testQueryView(self):
        view = self._setupQueryView()
        self.failUnless(view.validate())
        sample_types = view.getSampleTypes()
        projects = view.getProjects()
        view.doSearch()
        results = view.getResults()
        self.failUnless(len(results))

    def testQueryViewValidate(self):
        view = self._setupQueryView()
        good_dates = {'start_date':'10/24/2002',
                      'end_date':'10/25/2002'}
        bad_dates = {'start_date':'10/24/2002',
                     'end_date':'10/25/200'}
        #test bad collect date
        self.failUnless(view.request.form.get('message', None) is None)
        view.request.form['collect_dates'] = bad_dates
        view.request.form['entered_dates'] = good_dates
        self.failUnless(view.validate())
        self.failUnless(view.request.form.get('message', None) is not None)
        del view.request.form['message']
        #test bad entered date
        self.failUnless(view.request.form.get('message', None) is None)
        view.request.form['collect_dates'] = good_dates
        view.request.form['entered_dates'] = bad_dates
        self.failUnless(view.validate())
        self.failUnless(view.request.form.get('message', None) is not None)
        del view.request.form['message']
        #test good dates
        self.failUnless(view.request.form.get('message', None) is None)
        view.request.form['collect_dates'] = good_dates
        view.request.form['entered_dates'] = good_dates
        self.failUnless(view.validate())
        self.failUnless(view.request.form.get('message', None) is None)
        del view.request.form['collect_dates']
        del view.request.form['entered_dates']
        # test bad aliquot_id
        self.failUnless(view.request.form.get('message', None) is None)
        view.request.form['aliquot_ids'] = 'a'
        self.failUnless(view.validate())
        self.failUnless(view.request.form.get('message', None) is not None)
        del view.request.form['message']
        # test good aliquot_id
        self.failUnless(view.request.form.get('message', None) is None)
        view.request.form['aliquot_ids'] = '123'
        self.failUnless(view.validate())
        self.failUnless(view.request.form.get('message', None) is None)
        # test bad sample_id
        self.failUnless(view.request.form.get('message', None) is None)
        view.request.form['sample_ids'] = 'a'
        self.failUnless(view.validate())
        self.failUnless(view.request.form.get('message', None) is not None)
        del view.request.form['message']
        # test good sample_id
        self.failUnless(view.request.form.get('message', None) is None)
        view.request.form['sample_ids'] = '123'
        self.failUnless(view.validate())
        self.failUnless(view.request.form.get('message', None) is None)



    def testAdditionalDataQueryView(self):
        view = self._setupQueryView()
        view.request.form['ADDITIONAL_QUERY'] = True
        del view.request.form['search']
        view.request.form['subject_ids'] = []
        view.request.form['id_type'] = 'subject_id'
        view.request.form['collect_dates'] = {'start_date':'',
                                              'end_date':''}
        view.request.form['entered_dates'] = view.request.form['collect_dates']
        self.failUnless(view.validate())


    def testDeriveView(self):
        query_view = self._setupQueryView()
        project1 = self.projects['project1']['project']
        project1 = ProxyFactory(project1)
        query_view.doSearch()
        results = query_view.getResults()
        aliquot_ids = [results[0]['aliquots'][0]['id']]
        request = self.request
        request.form['derive'] = True
        request.form['aliquot_ids'] = aliquot_ids
        derive_edit = zapi.getView(project1, 'derive_aliquot_edit.html'
                                   , request)

        self.failUnless(query_view.splitorderive() == derive_edit())
        request.form['aliquots'] = [{'aliquot_count': u'',
                                     'concentration': u'',
                                     'concentration_unit': u'',
                                     'container_id': u'',
                                     'depleted': u'on',
                                     'id': aliquot_ids[0],
                                     'volume': u'',
                                     'volume_unit': u'',
                                     'weight': u'',
                                     'comment': u'',
                                     'weight_unit': u''}]
        do_edit = zapi.getView(project1, 'derive_aliquot_edit',
                               request)
        self.failUnless(do_edit())
        del request.form['aliquots']
        derive_view = zapi.getView(project1,'aliquot_derive.html',request)
        derive_view.request.form[u'DERIVE'] = True
        derive_view.request.form[u'field.name'] = u'test'
        derive_view.request.form[u'field.subject_type_id-empty-marker'] = u'1'
        derive_view.request.form[u'field.subject_type_id'] = unicode(self.other.name)
        derive_view.request.form[u'field.technician_id-empty-marker'] = u'1'
        derive_view.request.form[u'field.technician_id'] = u''
        derive_view.request.form[u'field.other_id'] = u''
        derive_view.request.form[u'field.subject_id'] = u''
        derive_view.request.form[u'field.comments'] = u''
        derive_view.request.form[u'field.collection_date'] = u'10/23/2003'
        derive_view.request.form[u'field.commit_message'] = u'test'
        derive_view.request.form[u'UPDATE_SUBMIT'] = u'UPDATE_SUBMIT'
        stype = self.projects['project1']['stypes'][0]
        derive_view.request.form['field.type_id'] = unicode(stype.name)
        samples = zapi.traverse(project1, 'samples')
        sample_count = len(samples)
        self.failUnless(derive_view())
        derive_view.update()
        self.failUnless(len(samples) == sample_count + 1)

    def testSplitView(self):
        from sip.aliquot import Aliquot
        vunit = self._addVolumeUnit()
        wunit = self._addWeightUnit()
        query_view = self._setupQueryView()
        project1 = self.projects['project1']['project']
        query_view.doSearch()
        results = query_view.getResults()
        request = self.request
        id = results[0]['aliquots'][0]['id']
        request.form['aliquots'] = [{'id': id,
                                     'split_count': '2'}]
        request.form['split'] = True
        split_view =  zapi.getView(project1, 'aliquot_split.html', request)
        self.failUnless(query_view.splitorderive())
        self.failUnless(split_view())
        aliquots = zapi.traverse(project1, 'aliquots')
        aliquot_count = len(aliquots)
        request.form['base_aliquots']=[{'id': id,
                                        'volume': 0.0,
                                        'volume_unit': vunit.id,
                                        'weight': 0,
                                        'weight_unit': wunit.id,
                                        'concentration': None,
                                        'concentration_unit': None,
                                        'container_id': None,
                                        'commit_message': '',
                                        'aliquot_count': None,
                                        'comment':None,
                                        'depleted': True}]
        request.form['aliquots']=[{'id': Aliquot.nextID(project1),
                                   'base_id': id,
                                   'volume': 2.0,
                                   'volume_unit': vunit.id,
                                   'weight': 3.0,
                                   'weight_unit': wunit.id,
                                   'concentration': None,
                                   'concentration_unit': None,
                                   'container_id': None,
                                   'aliquot_count': None,
                                   'comment': None,
                                   'commit_message': ''}]
        request.form['create_aliquots'] = True
        del request.form['split']
        results_view = zapi.getView(project1, 'split_results.html', request)
        self.failUnless(results_view())
        self.failUnless(len(aliquots) == aliquot_count + 1)
        aliquot = aliquots['sip.Aliquot.%d' % id]
        self.failUnless(aliquot.isEmpty())


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestQuery))
    return suite

if __name__ == '__main__':
    unittest.main()
