# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: test_schema.py 749 2005-06-28 22:21:49Z sidnei $
"""

import unittest
from zope.app import zapi
from zope.app.form.interfaces import IInputWidget
from zope.exceptions import NotFoundError
from zope.interface import implements, directlyProvides, directlyProvidedBy
from zope.schema import Text, Datetime, Bool
from zope.schema.interfaces import ConstraintNotSatisfied
from zope.security.checker import ProxyFactory
from zope.proxy import removeAllProxies

from sip.browser import api
from sip.ftests.base import Base
from sip.ftests.test_reportrole import SQLRole
from sip.interfaces.fields import ITALESConstrainedField, \
     TALESExpressionConstraint

class TestSchema(SQLRole, Base):
    level = 3
    _principal = 'lab_technician'
    _roles = ['sip.Report', 'sip.LabTechnician', 'sip.ProjectManager']
    full_setup = False
    use_test_user = False

    def _addSchema(self, setup_test_user=False):
        self.project = project = self._addProject()
        stypes = zapi.traverse(project, 'sampletypes')
        self.assertEquals(len(stypes), 0)
        stype = self._addSampleType(project)
        self.failUnless(stype.id == iter(stypes.values()).next().id)
        if setup_test_user:
            self._setupTestUser()
        schema = api.getSchemaFor(stype.id, project)
        return schema

    def _addSchemaWithFields(self, setup_test_user=False):
        schema = self._addSchema()
        field = Text(title=u'Mofo', required=False)
        schema.addField(u'mofo', field)
        return schema

    def _browserCommit(self, schema):
        url = str(zapi.getView(schema, 'absolute_url', self.request))
        self.request._traversed_names = url.split('/')[3:]
        # Check that we can add via browser by calling the view
        # directly. This should call update() by side-effect and
        # add the TALES field.
        editschema = zapi.getView(schema, 'editschema.html', self.request)
        editschema = ProxyFactory(editschema)
        editschema()
        self.request.form.clear()

    def _addFieldTTW(self, schema, **kwargs):
        form=self.request.form
        for k,v in kwargs.items():
            form['field.'+k]=v
        form.setdefault('field.required', 'off')
        form.setdefault('field.required.used', True)
        form.setdefault('field.usetales', 'on')
        form.setdefault('field.usetales.used', True)
        form.setdefault('field.type', u'Integer Field')
        form.setdefault('field.id', u'Foo')
        form.setdefault('field.title', u'A title')
        form.setdefault('field.description', u'Generated Description')

        form['ADD_FIELD'] = True
        form['UPDATE_SUBMIT'] = True
        self._browserCommit(schema)

    def _delFieldTTW(self, schema, **kwargs):
        form=self.request.form
        form['DELETE'] = True
        form['ids'] = kwargs['ids']
        self._browserCommit(schema)

    def test_removeFieldFromSchema(self):
        schema = self._addSchemaWithFields()
        schema.removeField(u'mofo')
        self.failUnlessRaises(NotFoundError, zapi.traverse, *(schema, 'mofo'))
        self._addFieldTTW(schema, type=u'Text Field', id=u'mofo', required=False,
                          title=u'A title')
        self._delFieldTTW(schema, ids=['mofo',])
        self.failUnlessRaises(NotFoundError, zapi.traverse, *(schema, 'mofo'))

    def test_system_user_traverse_schema_and_manage(self):
        schema = self._addSchemaWithFields()
        field = zapi.traverse(schema, 'mofo')
        view = zapi.getViewProviding(field, IInputWidget, self.request)
        view = ProxyFactory(view)
        self.failUnless(view())

    def test_sql_user_traverse_schema_and_manage(self):
        schema = self._addSchemaWithFields(setup_test_user=True)
        # Add another field -- which happens to be a
        # TALES field -- the browser way
        self._addFieldTTW(schema, type='Integer Field', title=u'integer',
                          id=u'integer', description=u'An Integer Field',
                          required='off', usetales='on')

        # Add a third field, the "normal" way, but with a constraint
        field = Text(title=u'nickname', required=True)
        #directlyProvides(field, directlyProvidedBy(field),
        #                 ITALESConstrainedField)
        #mofo.constraint = TALESConstraint(u'python:len(value)')
        schema.addField(u'nickname', field)
        schema.constrain(u'nickname', u'python:len(value)')
        # Check that we can get to the first field
        field = zapi.traverse(schema, 'mofo')
        # And call it's edit view
        view = zapi.getViewProviding(field, IInputWidget, self.request)
        view = ProxyFactory(view)
        self.failUnless(view())
        # Then check that we can get to the second field
        field = schema.get('integer')
        self.failUnless(ITALESConstrainedField.providedBy(field))
        field = zapi.traverse(schema, 'integer')
        self.failUnless(ITALESConstrainedField.providedBy(field))
        # And call it's edit view
        view = zapi.getView(field, 'edit.html', self.request)
        view = ProxyFactory(view)
        # And call it's TALES constraint edit view
        view = zapi.getView(field, 'talesconstraint.html', self.request)
        view = ProxyFactory(view)
        # Now check that we can get to the third field
        field = schema.get('nickname')
        self.failUnless(ITALESConstrainedField.providedBy(field))
        field = zapi.traverse(schema, 'nickname')
        self.failUnless(ITALESConstrainedField.providedBy(field))

    def test_schema_name_constraint(self):
        schema = self._addSchemaWithFields(setup_test_user=True)
        self.assertRaises(ConstraintNotSatisfied,
                         self._addFieldTTW,
                         schema, type='Integer Field', title=u'integer',
                         id=u'name', description=u'An Integer Field',
                         required='off', usetales='on')

    def test_tales_constraint(self):
        constraint = TALESExpressionConstraint
        self.failUnless(constraint('u in v'))
        self.failUnless(constraint('argh'))
        self.failUnless(constraint("b.startswith('bla')"))
        self.failIf(constraint('u in v('))
        self.failIf(constraint('u, in v'))
        self.failIf(constraint('u in'))


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestSchema))
    return suite

if __name__ == '__main__':
    unittest.main()
