# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/


from zope.interface import Interface, Attribute
from zope.schema import TextLine, Text, Bool, Choice, Date, Float
from sip.i18n import _
from sip.interfaces import ISQLSchema, Unique, FieldFactory, ISIPSchema
from sip.descriptors import BoundCallableDescriptor

class IContactInfo(ISIPSchema):
    """ information for shipment contact """

    firstname = TextLine(
        title = _('First Name'),
        description = _('The contacts first name'),
        max_length = 50,
        required = True)

    lastname = TextLine(
        title = _('Last Name'),
        description = _('The contacts last name'),
        max_length = 50,
        required = True)

    institution = TextLine(
        title = _('Institution'),
        description = _('Institution contact works for'),
        max_length = 100,
        required = True)

    address1 = TextLine(
        title = _('Address Line 1'),
        max_length = 100,
        required = True)

    address2 = TextLine(
        title = _('Address Line 2'),
        max_length = 100,
        required = False)

    city = TextLine(
        title = _('City'),
        description = _('Contact City'),
        max_length = 50,
        required = False)

    state = TextLine(
        title = _('State'),
        description = _('Contact State'),
        max_length = 50,
        required = False)

    zipcode = TextLine(
        title = _('Zip Code'),
        description = _('Contact Zip Code'),
        max_length = 10,
        required = False)

    country = TextLine(
        title = _('Country'),
        description = _('Contact Country'),
        max_length = 50,
        required = False)

    phone = TextLine(
        title = _('Phone Number'),
        description = _('Contact Phone number'),
        max_length = 20,
        required = False)

    fax = TextLine(
        title = _('Fax Number'),
        description = _('Contact Fax Number'),
        max_length = 20,
        required = False)

    email = TextLine(
        title = _('Email Address'),
        description = _('Contact Email Address'),
        max_length = 50,
        required = False)

    project = TextLine(
        title = _('Project Title'),
        description = _('Title of project samples will be used for'),
        max_length = 100,
        required = True)

    objective = Text(
        title = _('Study Objective'),
        description = _('What the project wishes to learn'),
        required = False)


class IShippingInfo(ISIPSchema):

    hand_delivered = Bool(
        title = _('Hand Delivered'),
        description = _('Delivered by Lab Personnel'),
        required = False)

    newcontainer = Bool(
        title = _('New Container'),
        description = _('Was this shipment sent in a new container'),
        required = False)

    carrier_id = Choice(
        title = _('Carrier'),
        description = _('Shipping Carrier'),
        readonly = False,
        required = False,
        vocabulary = 'sip.vocab.carriers')

    account_number = TextLine(
        title = _('Account Number'),
        description = _('Shipping Account Number'),
        max_length = 50,
        required = False)

    housenumber = TextLine(
        title = _('House Number'),
        description = _('House Account Number'),
        required = False)

    tracking_number = TextLine(
        title = _('Tracking Number'),
        description = _('Shipping Tracking Number'),
        max_length = 50,
        required = False)

    temp_method_id = Choice(
        title = _('Temperature Maintenance Method'),
        description = _('Method used to maintain temperature during shipping'),
        readonly = False,
        vocabulary = 'sip.vocab.tempmethod',
        required = False)

    technician_id = Choice(
        title=_('Shipping Technician'),
        description=_('The technician who sent this shipment.'),
        readonly=False,
        required=False,
        vocabulary='sip.vocab.project_technicians')

    instructions = Text(
        title = _('Instructions'),
        description = _('Shipping Instructions'),
        max_length = 255,
        required = False)


class ICarrier(ISQLSchema):

    name = FieldFactory(
        TextLine,
        title = _('Name'),
        description = _('Carrier Name'),
        max_length = 20,
        required = True,
        constraint=BoundCallableDescriptor(
            Unique(className='sip.Carrier', columnName='name'))
        )

class IFundingSource(ISQLSchema):

    name = FieldFactory(
        TextLine,
        title = _('Name'),
        description = _('Funding Source Name'),
        max_length = 20,
        required = True,
        constraint=BoundCallableDescriptor(
            Unique(className='sip.FundingSource', columnName='name'))
        )

    federal = Bool(
        title = _('Federal'),
        description = _('Is this source federal?'),
        required=False
        )

class IFundingInfo(ISIPSchema):

    shipment_id = Attribute('Shipment')

    funding_source_id = Choice(
        title = _('Funding Source'),
        description = _('Funding Source'),
        readonly=False,
        required=False,
        vocabulary = 'sip.vocab.fundingsources'
        )

    grant_number = TextLine(
        title = _('Grant Number'),
        description = _('Grant Number'),
        max_length = 50,
        required = False
        )

    cost = Float(
        title = _('Annual Direct Cost'),
        description = _('Annual Direct Cost'),
        required = False
        )

    agency = TextLine(
        title = _('Agency or Company Name'),
        description = _('Agency or Company Name'),
        max_length = 50,
        required = False
        )

    principal = TextLine(
        title = _('Principal Investigator'),
        description = _('Principal Investigator'),
        max_length = 50,
        required = False
        )

    state = TextLine(
        title = _('State'),
        description = _('State the request initiates from'),
        max_length = 20,
        required = False
        )

    enddate = Date(
        title = _('End Date'),
        description = _('Date requesting grant ends'),
        required = False
        )

    fee = Float(
        title = _('Fee'),
        description = _('Amount charged to requesting grant or company'),
        required = False
        )

class IShippingWizard(IContactInfo, IShippingInfo):
    """combine everything so that we can create wizard"""

    name = TextLine(
        title = _('Name'),
        description = _('Shipment Name'),
        max_length = 255,
        required = True)

    shipdate = Date(
        title = _('Ship Date'),
        description = _('Date shipment sent'),
        required = False)

    receivedate = Date(
        title = _('Receive Date'),
        description = _('Date shipment was received'),
        required = False)

class ITempMethod(ISQLSchema):
    """ Tempereature maintenance method """

    name = TextLine(
        title = _('Name'),
        description = _('Method Name'),
        max_length = 255,
        required = True)
