# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: fields.py 749 2005-06-28 22:21:49Z sidnei $
"""

from datetime import date
from sqlobject import *
from sqlos import SQLOS
from zope.interface import implements, directlyProvides, directlyProvidedBy
from zope.app import zapi
from sip.descriptors import BoundCallableDescriptor, FakePropertyDescriptor
from sip.interfaces.schema import ISchemaField
from sip.browser import api
from sip.wrapper import Delegate
from sip.interfaces.fields import ITALESConstrainedField
from sip.browser.schema import TALESConstraint

class SchemaField(SQLOS):

    implements(ISchemaField)

    _idName='schema_field_id'
    _columns = [
        ForeignKey('SampleTypeSchema', name='schema_id', cascade=False),
        StringCol('name', notNull=1),
        StringCol('type', notNull=1),
        StringCol('title', notNull=0, default=""),
        StringCol('description', notNull=0, default=""),
        BoolCol('required', notNull=1, default=False),
        BoolCol('readonly', notNull=1, default=False),
        StringCol('default_value', notNull=0, default=""),
        IntCol('field_order', notNull=1, default=0),
        BoolCol('tales', notNull=1, default=False),
        StringCol('tales_expression', notNull=1, default=""),
        DateCol('start_date', notNull=1, default=date.today),
        DateCol('end_date', notNull=0, default=None),
        ]

    order = FakePropertyDescriptor('order', 'field_order')
    default = FakePropertyDescriptor('default', 'default_value')
    expression = FakePropertyDescriptor('expression', 'tales_expression')

def toField(item, context):
    mparts = item.type.split('.')
    mname = '.'.join(mparts[:-1])
    kname = mparts[-1]
    module = __import__(mname, globals(), locals(), mname)
    klass = getattr(module, kname)
    kw = {}
    for k in ('title', 'description', 'required', 'readonly',
              'default_value', 'tales', 'tales_expression'):
        value = getattr(item, k)
        if isinstance(value, basestring):
            value = unicode(value)
        kw[k] = value
    name = unicode(item.name)
    kw['__name__'] = name
    tales = kw.get('tales')
    del kw['tales']
    if tales:
        # We use a delegate here as well, as the
        # constraint is a subobject, thus we can't catch
        # the attribute assignement at that level
        constraint = Delegate(TALESConstraint(), item, ('expression',))
        kw['constraint'] = constraint
    tales_expression = kw.get('tales_expression')
    del kw['tales_expression']
    if tales and tales_expression:
        kw['constraint'].expression = tales_expression
    default = kw.get('default_value')
    del kw['default_value']
    field = klass(**kw)
    if default:
        field_type, default = api.convertFieldAndValue(field, default)
        field.default = default
    if tales and tales_expression:
        directlyProvides(field, directlyProvidedBy(field),
                         ITALESConstrainedField)
    field.order = item.field_order
    field = api.contained(field, context, name)
    return Delegate(field, item, tuple(kw.keys()) + ('order', 'default'))

class FieldsStorage:

    def __init__(self, context, factoryName='sip.SchemaField'):
        self.context = context
        self.factory = api.factory(factoryName, context=context)

    def __getitem__(self, name):
        name = str(name)
        query = 'name = %s and schema_id = %d' % (
            self.factory.sqlrepr(name), self.context.id)
        result = self.factory.select(clause=query)
        if not result.count():
            raise KeyError, name
        item = result[0]
        return toField(item, self.context)

    def get(self, name, default=None):
        try:
            item = self[name]
        except KeyError:
            return default
        return item

    def __setitem__(self, name, value):
        kw = {}
        kw['name'] = name
        kw['schema_id'] = self.context.id
        for k in ('title', 'description', 'required',
                  'readonly'):
            kw[k] = getattr(value, k)
        kw['field_order'] = value.order
        kw['default_value'] = str(value.default)
        klass = value.__class__
        kw['type'] = "%s.%s" % (klass.__module__, klass.__name__)
        if ITALESConstrainedField.providedBy(value):
            kw['tales'] = True
            kw['tales_expression'] = value.constraint.expression
        self.factory(**kw)

    def __delitem__(self, name):
        item = self[name]
        item.destroySelf()

    def items(self):
        query = 'schema_id = %d' % self.context.id
        result = list(self.factory.select(clause=query))
        if not result:
            return iter([])
        c = lambda p: toField(p, self.context)
        items = map(c, result)
        items = [(f.__name__, f) for f in items]
        return items

    def keys(self):
        return [unicode(k) for k, v in self.items()]

    def __iter__(self):
        return iter(self.keys())

    def values(self):
        for k, v in self.items():
            yield v

    def __contains__(self, name):
        return self.get(name, None) is not None

    def __len__(self):
        return len(self.keys())

    def constrain(self, name, constraint):
        name = str(name)
        query = 'name = %s and schema_id = %d' % (
            self.factory.sqlrepr(name), self.context.id)
        result = self.factory.select(clause=query)
        if not result.count():
            raise KeyError, name
        item = list(result)[0]

        if ITALESConstrainedField.providedBy(constraint):
            constraint = constraint.expression
        if constraint is None:
            item.tales = False
            item.tales_expression = ''
        else:
            item.tales = True
            item.tales_expression = constraint
