# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: source.py 758 2005-06-28 23:02:39Z sidnei $
"""

__metaclass__ = type

from itertools import imap

from zope.app.security.settings import Allow, Deny, Unset
from zope.app import zapi
from zope.event import notify
from zope.app.event.objectevent import ObjectModifiedEvent

from sqlos.interfaces import ISQLObject
from sip.browser import api
from sip.browser.api import uniqueId
from sip.source import Singleton
from zope.app.location import Location

_pmap = {Allow : 1,
        Deny : 0,
        Unset : -1}
_imap = {1 : Allow,
        0 : Deny,
        -1 : Unset}

class SecurityCache(dict): pass

securityCache = SecurityCache()

class ResultSet(dict):

    def __init__(self, factory, keys=None,
                 value=None, location=None,
                 context=None):
        dict.__init__(self)
        self._factory = factory
        if keys:
            self._keys = keys
        self._value = value
        self._location = location
        self._context = context

    def __getitem__(self, name):
        item = dict.__getitem__(self, name)
        return _imap[item.value]

    def get(self, name, default):
        try:
            item = self[name]
        except KeyError:
            return default
        else:
            return item

    def __setitem__(self, name, value):
        if ISQLObject.providedBy(value):
            item = value
        else:
            try:
                item = dict.__getitem__(self, name)
            except KeyError:
                factory = api.factory(self._factory, context=self._context)
                kw = {self._keys[1]:name}
                kw[self._keys[0]] = self._value
                kw['location'] = self._location
                item = factory(**kw)
            value = _pmap[value]
            item.value = value
            notify(ObjectModifiedEvent(item))
        dict.__setitem__(self, name, item)

    def __delitem__(self, name):
        item = dict.__getitem__(self, name)
        # XXX Should really be ObjectDeletedEvent or a similar one,
        # but we can go away with this, at least for now.  We do this
        # to trigger the clearing of the securityCache objects we are
        # using, which is done with an event.
        notify(ObjectModifiedEvent(item))
        item.destroySelf()
        dict.__delitem__(self, name)

    def values(self):
        return [v for v in dict.itervalues(self)]

    def items(self):
        return [(k, _imap[v.value]) for k, v in dict.iteritems(self)]

class Source(Location, Singleton):

    def __init__(self, context, className=None, keys=None):
        self.context = context
        if className:
            self.className = className
        if keys:
            self._keys = keys

    def getQuery(self):
        clause = 'location = %r' % str(uniqueId(self.context))
        return clause

    def keys(self):
        r = {}
        for k, v in self.items():
            r[k] = None
        return r.keys()

    def __iter__(self):
        return iter(self.keys())

    def values(self):
        for k, v in self.items():
            yield v

    def items(self):
        factory = api.factory(self.className, context=self)
        c = lambda p: (getattr(p, self._keys[0]), p)
        return imap(c, factory.select(self.getQuery()))

    def __getitem__(self, name):
        cache = securityCache
        path = zapi.getPath(self.context)
        key = (self, (self.className, name), path)
        results = cache.get(key)
        if results is not None:
            return results
        location = uniqueId(self.context)
        factory = api.factory(self.className, context=self)
        results = cache[key] = ResultSet(factory=self.className,
                                         keys=self._keys,
                                         value=str(name),
                                         location=location,
                                         context=self.context)
        try:
            clause = "%s = %s" % (getattr(factory.q, self._keys[0]), factory.sqlrepr(name))
        except AttributeError:
            return []
        clause = "%s and %s" % (self.getQuery(), clause)
        for v in factory.select(clause=clause):
            results[getattr(v, self._keys[1])] = v
        return results

    def get(self, name, default=None):
        try:
            item = self[name]
            return item
        except KeyError:
            return default

    def __contains__(self, name):
        return self.get(name, None) is not None

    def __len__(self):
        return len(self.keys())

    def __delitem__(self, name):
        results = self[name]
        for v in results.values():
            v.destroySelf()

# Register our cleanup with Testing.CleanUp to make writing unit tests simpler.
from zope.testing.cleanup import addCleanUp
addCleanUp(securityCache.clear)
del addCleanUp
