# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: interfaces.py 759 2005-06-28 23:45:42Z sidnei $
"""

from zope.interface import Interface, Attribute, invariant
from zope.schema import Date, TextLine, Bool
from sip.interfaces import ISIPSchema, ISQLSchema
from sip.interfaces.source import ISource
from sip.i18n import _
from zope.schema.interfaces import ValidationError
from zope.app.container.constraints import ItemTypePrecondition
from sip.interfaces import FieldFactory, Unique
from sip.descriptors import BoundCallableDescriptor
from datetime import datetime

class ReceiveDateError(ValidationError):
    __doc__ = _("""Receive Date < Ship Date""")

class FutureDateError(ValidationError):
    __doc__ = _("""This date cannot be from the future""")

def date_invariant(obj):
    if obj.receivedate and obj.shipdate and obj.receivedate < obj.shipdate:
        raise ReceiveDateError

def noFutureDates(value):
    today = datetime.date(datetime.today())
    if value > today:
        raise FutureDateError()
    return True

class IShipment(ISIPSchema):

    invariant(date_invariant)

    name = FieldFactory(
        TextLine,
        title = _('Name'),
        description = _('Shipment Name'),
        constraint = BoundCallableDescriptor(
            Unique(className='sip.Shipment', columnName='name')),
        required = True)

    requestdate = Date(
        title = _('Request Date'),
        description = _('Date request was made'),
        required = False)

    planneddate = Date(
        title = _('Planned Ship Date'),
        description = _('Date we plan to ship on'),
        required = False)

    shipdate = Date(
        title = _('Ship Date'),
        description = _('Date shipment sent'),
        constraint = noFutureDates,
        required = False)

    receivedate = Date(
        title = _('Receive Date'),
        description = _('Date shipment was received'),
        constraint = noFutureDates,
        required = False)

    returndate = Date(
        title = _('Container Return Date'),
        description = _('Date shipping container was returned'),
        constraint = noFutureDates,
        required = False)

    infectious = Bool(
        title = _('Infectious Substance'),
        description = _('Does the shipment contain an infectious substance'),
        required = False)

    biohazard = Bool(
        title = _('Biohazard'),
        description = _('Is the shipment a biohazard'),
        required = False)

    contact_info_id = Attribute('The contact info for the shipment')

    shipment_info_id = Attribute('The shipping info for the shipment')

    project_id = Attribute('Teh project')


class IContainerShipment(ISIPSchema):

    container_id = Attribute('The container for this shipment')

    shipment_id = Attribute('The shipment')

class IShipmentDirection(Interface):
    """ A marker interface for selecting shipments according
    to the direction of the shipment (incoming/outgoing).
    """
    name = TextLine(title=u'Name')

class IShipmentState(ISource):
    """ A marker interface for selecting shipments according
    to the state.
    """
    name = TextLine(title=u'Name')


class IOutgoingShipmentState(IShipmentState):
    """ A marker interface for state of
    outgoing shipments.
    """

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(IShipment)


class IIncomingShipmentState(IShipmentState):
    """ A marker interface for state of
    incoming shipments.
    """

class IShipmentAliquots(Interface):
    """ A marker interface for aliquots contained in a shipment """
