# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

__metaclass__ = type

from sip.descriptors import SecurityDescriptor, ClassDescriptor
from zope.event import notify
from zope.app.event.objectevent import ObjectModifiedEvent

# Put those attributes in a list so its easier to add/remove
# when needed.
unproxied_attrs = ['__proxied__',
                   '__mirror__',
                   '__attlist__',
                   '__Security_checker__',
                   ]

class Delegate:
    """Delegates attribute access between two objects so
    that access and modification of attributes on the first
    'proxied' object are directed to the 'mirror' object.
    """

    __class__ = ClassDescriptor()
    __Security_checker__ = SecurityDescriptor()

    def __init__(self, proxied, mirror, attlist):
        self.__proxied__ = proxied
        self.__mirror__ = mirror
        self.__attlist__ = attlist

    def __getattribute__(self, name):
        if name in unproxied_attrs:
            v = object.__getattribute__(self, name)
            # Handle descriptors here, eg: __Security_checker__
            # is a descriptor the Delegate object.
            if hasattr(v, '__get__'):
                # We should call it only if it came from the class,
                # otherwise its a descriptor being used as an instance
                # attribute, so just return it.
                if (hasattr(self, '__class__') and
                    getattr(self.__class__, name) is v):
                    return v.__get__(self, type(self))
            return v
        proxied = self.__proxied__
        mirror = self.__mirror__
        if not hasattr(proxied, name) and hasattr(mirror, name):
            proxied = mirror
        v = getattr(proxied, name)
        # And also handle descriptors for the proxied object,
        # but using the proxied object on __get__ calls.
        if hasattr(v, '__get__'):
            # We should call it only if it came from the class,
            # otherwise its a descriptor being used as an instance
            # attribute, so just return it.
            if (hasattr(proxied, '__class__') and
                getattr(proxied.__class__, name) is v):
                return v.__get__(proxied, type(proxied))
        return v

    def __setattr__(self, name, v):
        if name in unproxied_attrs:
            return object.__setattr__(self, name, v)
        attlist = self.__attlist__
        proxied = self.__proxied__
        mirror = self.__mirror__
        setattr(proxied, name, v)
        notify(ObjectModifiedEvent(proxied))
        if name in attlist and not (getattr(mirror, name, None) == v):
            setattr(mirror, name, v)
            notify(ObjectModifiedEvent(mirror))

    def __delattr__(self, name):
        if name in unproxied_attrs:
            return object.__delattr__(self, name)
        attlist = self.__attlist__
        proxied = self.__proxied__
        mirror = self.__mirror__
        delattr(proxied, name, v)
        if name in attlist:
            delattr(mirror, name, v)

    def __call__(self, *args, **kw):
        return self.__proxied__(*args, **kw)
